SRCHROM		    equ 0xF0
RDROM		    equ 0x33
MTCHROM		    equ 0x55
SKPROM		    equ 0xCC
DS18B20_ID	    equ 0x28

DelayUsLocal:MACRO T
    errorlevel -207				    ; for code justifying
    
    if T >= (256*197122)+7
	error "Too large T value"
	exitm
    endif
    
    local t = (T)
    local w

    w = (((t)-C1L) / T1L)
    
    if w > 0
	MOVLW	w
	MOVWF	delay1_cnt
	CALL	delayUsFLocal
	t = (((t)-C1L) % T1L)
    endif
    
    w = t
    
    while w > 1
	GOTO	$+1
	w -= 2
    endw
    
    while w > 0
	NOP
	w -= 1
    endw
    
    errorlevel +207				    ; for code justifying
    ENDM
#define FCL (1) ; fclk [MHz]
    
#define C1L  (5/FC)
#define T1L  (3/FC)
delayUsFLocal:
    DECFSZ	delay1_cnt,F
    GOTO	delayUsFLocal
    RETLW	0

READ_TYPE_DS_LAB:
seq1_proc1:    
    CLRF	crc_reg
    CALL        OW_RESET_FUN			    ; Send Reset Pulse and read for Presence Detect Pulse
    
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc2:
#endif
    
    MOVLW	0x33				    ; Read On-Board ROM
    CALL	DSTXBYTE_FUN			    ; send command to DS    
    
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc3:
#endif

    CALL        DSRXBYTE_FUN			    ; Byte 0 from On-Board ROM - device ID
    
    MOVF        IOBYTE,W			    ; Save device_id
    MOVWF       tmp1
    MOVLW	7				    ; Reading remain seven bytes from On-Board ROM
    MOVWF	cnt2
READ_TYPE_DS_LAB_loop1:
    
#ifdef SEQUENCE    
    GOTO	seq1_ret  
seq1_proc4:
    incf	seq1_ptr,F
#endif    
    CALL        DSRXBYTE_FUN
    
    DECFSZ	cnt2,F
    GOTO	READ_TYPE_DS_LAB_loop1
    BCF		main_state, MATCH_CRC_FLAG
    MOVF	crc_reg,f
    BTFSC	STATUS,Z			    ; If crc_reg is zero then MATCH_CRC_FLAG is true
    BSF		main_state, MATCH_CRC_FLAG    
CONFIG_DS_LAB:
#ifdef SEQUENCE    
    DECF	seq1_ptr,F			    ; because end of the loop, get next sequence
    GOTO	seq1_ret
seq1_proc5:
#endif
    CALL	OW_RESET_FUN			    ; Send Reset Pulse and read for Presence Detect Pulse
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc6:
#endif
    MOVLW	0xCC				    ; Skip ROM
    CALL	DSTXBYTE_FUN
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc7:
#endif
    MOVLW	0x4E				    ; Write scratchpad
    CALL	DSTXBYTE_FUN			    ; Send function command    
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc8:
#endif
    CALL	DSTXBYTE_FUN			    ; Send TL
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc9:
#endif
    CALL	DSTXBYTE_FUN			    ; Send TH
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc10:
#endif
    MOVLW	0x1F				    ; 0b00011111 - 9bit resolution
    CALL	DSTXBYTE_FUN			    ; Send 3rd scratch byte (conf)    
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc11:
#endif
    
CONVERT_TEMP_LAB:
    CALL	OW_RESET_FUN			    ; Send Reset Pulse and read for Presence Detect Pulse
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc12:
#endif
    
    MOVLW	0xCC				    ; Skip ROM
    CALL	DSTXBYTE_FUN    
#ifdef SEQUENCE
    GOTO	seq1_ret
seq1_proc13:
#endif
    
    MOVLW	0x44
    CALL	DSTXBYTE_FUN			    ; Send T conv    
    
    OwOutH					    ; Strong pull-up on - not required while non-parasite supply
#ifndef SEQUENCE
    DelayUs	93750				    ; wait 141205us
#else
    MOVLW	.47+.1
    MOVWF	ds_conv_del_cnt
    GOTO	seq1_ret
seq1_proc14:
    DECF	ds_conv_del_cnt,F
    BTFSC	STATUS,Z
    GOTO	seq1_proc14_end
    incf	seq1_ptr,F
    GOTO	seq1_ret			    ; return if not acomplished
seq1_proc14_end:
#endif    
    BSF		tris_masq_c,DS18B20_PIN
    MOVF	tris_masq_c,W
    TRIS	DS18B20_PORT
    

GET_TEMP_LAB:
    CLRF	crc_reg
    CALL	OW_RESET_FUN			    ; Send Reset Pulse and read for Presence Detect Pulse
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc15:
#endif  
    
    MOVLW	0xCC
    CALL	DSTXBYTE_FUN
    
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc16:
#endif      
    
    MOVLW	0xBE				    ; Read scratchpad    
    CALL	DSTXBYTE_FUN			    ; Send command to DS
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc17:
#endif      
    CALL	DSRXBYTE_FUN			    ; Byte 0 - Temp LSB
    
#ifdef SEQUENCE    
    GOTO	seq1_ret
seq1_proc18:
#endif      
    MOVF	IOBYTE,W
    MOVWF	TLSB				    ; LSB temperature
    CALL	DSRXBYTE_FUN			    ; Byte 1 - Temp MSB
    
    MOVF	IOBYTE,W
    MOVWF	TMSB				    ; MSB temperature
    MOVLW	.7
    MOVWF	cnt2
gt_loop:
#ifdef SEQUENCE
    GOTO	seq1_ret
seq1_proc19:
    incf	seq1_ptr,F
#endif
    CALL        DSRXBYTE_FUN
    
    DECFSZ	cnt2,F
    GOTO	gt_loop
    BCF		main_state, MATCH_CRC_FLAG
    MOVF	crc_reg,f		    
    BTFSC	STATUS,Z			    ;if crc_reg is zero then MATCH_CRC_FLAG is true
    BSF		main_state, MATCH_CRC_FLAG 
    GOTO	DS18B20_TEMP_CONV_LAB
DS18B20_TEMP_CONV_LAB_ret:
    
#ifdef SEQUENCE   
    DECF	seq1_ptr,F
    GOTO	seq1_ret
#else
    GOTO	READ_TYPE_DS_LAB_END
#endif

DSRXBYTE_FUN:
    MOVLW       8				    ; Byte read is stored in IOBYTE
    MOVWF       tmp0				    ; Set tmp0 equal to 8 to count the bits
dsrx_loop:
    BCF		DS18B20_PORT, DS18B20_PIN
    BCF		tris_masq_c, DS18B20_PIN
    MOVF	tris_masq_c,W
    TRIS	DS18B20_PORT
    
    GOTO	$+1
    GOTO	$+1
    GOTO	$+1
    BSF		tris_masq_c,DS18B20_PIN
    MOVF	tris_masq_c,W
    TRIS	DS18B20_PORT

    GOTO	$+1
    GOTO	$+1
    GOTO	$+1
    GOTO	$+1
    MOVF        DS18B20_PORT,W			    ; Read DQ
    ANDLW	(1 << DS18B20_PIN)		    ; Mask off the DQ bit
    MOVWF	cnt1
    MOVLW	.255
    ADDWF	cnt1,F
    RRF         IOBYTE,F			    ; Shift C into IOBYTE
    DelayUsLocal	38
    
    DECFSZ      tmp0,F				    ; Decrement the bit counter
    GOTO        dsrx_loop
    MOVF	IOBYTE,W			    ; copy IOBYTE...
    GOTO	CRC_CALC_LAB			    ; and append it to crc calculation 
CRC_CALC_LAB_ret:
    RETLW	0

OW_RESET_FUN:
    BSF		tris_masq_c,DS18B20_PIN
    MOVF	tris_masq_c,W
    TRIS	DS18B20_PORT

    DelayUsLocal	100			    ; wait 1403us
    
    BCF		main_state, PRESENCE_FLAG	    ; Clear DS presence flag
    BCF		DS18B20_PORT, DS18B20_PIN
    BCF		tris_masq_c, DS18B20_PIN
    MOVF	tris_masq_c,W
    TRIS	DS18B20_PORT
    
    DelayUsLocal	353			    ; wait 353us

    BSF		tris_masq_c,DS18B20_PIN
    MOVF	tris_masq_c,W
    TRIS	DS18B20_PORT
    
    DelayUsLocal	52			    ; wait 52us
    
    BTFSS       DS18B20_PORT,DS18B20_PIN	    ; Read for a PD Pulse
    BSF		main_state,PRESENCE_FLAG	    ; Set DS presence flag
    MOVLW	94
    MOVWF	delay1_cnt
owrst4:
    DECFsz	delay1_cnt,F
    GOTO	owrst4
    
    RETLW 0

    
DSTXBYTE_FUN:
    MOVWF	IOBYTE				    ; Byte to send starts in W and copy to IOBYTE
    MOVLW	8
    MOVWF	tmp0				    ; Set tmp0 equal to 8 to count the bits
dstx_loop:
    BCF		DS18B20_PORT, DS18B20_PIN
    BCF		tris_masq_c, DS18B20_PIN
    MOVF	tris_masq_c,W
    TRIS	DS18B20_PORT
    
    CLRW    
    MOVLW	gpio_masq_c & ~(1 << DS18B20_PIN)
    RRF		IOBYTE,F
    BTFSC	STATUS,C			    ; Check the LSB of IOBYTE for 1 or 0
    MOVLW	gpio_masq_c | (1 << DS18B20_PIN)
    TRIS	DS18B20_PORT
    DelayUsLocal	45			    ; wait 45us

    BSF		tris_masq_c,DS18B20_PIN
    MOVF	tris_masq_c,W
    TRIS	DS18B20_PORT
    
    nop
    nop
    
    DECFSZ	tmp0,F				    ; Decrement the bit counter
    GOTO	dstx_loop
    
    RETLW	0

DS18B20_TEMP_CONV_LAB:
    MOVF	TMSB,W
    MOVWF	cnt1
    MOVF	TLSB,W
    MOVWF	cnt2
    RLF		cnt1,F				    ; Temperature 18B20 sensor conversion
    RLF		cnt1,F
    RLF		cnt1,F
    RLF		cnt1,F
    RLF		cnt1,F
    RRF		cnt2,F
    RRF		cnt2,F
    RRF		cnt2,F
    MOVF	cnt1,W				    ; temp = ((th << 5) | (tl >> 3)) - (tmin * 2)
    ANDLW	b'11100000'
    MOVWF	temp_real
    MOVF	cnt2,W
    ANDLW	b'00011111'

    IORWF	temp_real,F
    GOTO	DS18B20_TEMP_CONV_LAB_ret
    
CRC_CALC_LAB: 
    MOVF	IOBYTE,W
    MOVWF	crc_input
    MOVLW	.8
    MOVWF	cnt1
ccf_loop:
    MOVF	crc_reg,W
    BCF		STATUS,C
    RRF		crc_reg,F
    XORWF	crc_input,W
    ANDLW	0x01
    BTFSC	STATUS,Z
    GOTO	ccf_next_byte
    MOVLW	0x8C
    XORWF	crc_reg,F
ccf_next_byte:        
    RRF		crc_input,F
    DECF	cnt1,F
    BTFSS	STATUS,Z
    GOTO	ccf_loop
ccf_end:
    GOTO	CRC_CALC_LAB_ret
    
    